package fr.inra.moulon.svgutils.svgspectrum;

import java.util.ArrayList;

import fr.inra.moulon.svgutils.elements.SvgScript;
import fr.inra.moulon.svgutils.elements.SvgText;

/**
 * SvgSpectrum free java library to draw annotated MS spectrum Copyright (C)
 * 2009 Olivier Langella
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * @author Olivier Langella <olivier.langella@moulon.inra.fr
 * 
 */

public class DynamicSvgSpectrumDocument extends SvgSpectrumDocument {

	public DynamicSvgSpectrumDocument(float width, float height)
			throws Exception {
		super(width, height);

		SvgText labelMass = this.getPeaksSvgGroup().newText(" ");
		labelMass.setX(50);
		labelMass.setY(50);
		labelMass.setId("mzInfo");
		SvgScript javascriptElement = this.getSvgRoot().newScript();

		// console.log('x'+cursor.x+' y'+cursor.y);
		String code = "var mzInfo;var cursorOut={x:0, y:0};"
				+ "function initSvg() {mzInfo = document.getElementById(\"mzInfo\");"
				+ "mzInfo.setPosition = function (cursor) {mzInfo.setAttributeNS(null,'x', cursor.x);mzInfo.setAttributeNS(null,'y', cursor.y);"
				+ "};"
				+ "hideMass();}"
				// + this.writeGetPositionFunction()
				// + this.writeGetScrollPositionFunction()
				+ "function getEvtPosition(evt) {"
				+ "var cursor = {x:0, y:0};"
				+ "cursor.x = evt.target.getBBox().x;"
				+ "cursor.y = evt.target.getBBox().y;"
				// +
				// "console.log('svgRef.offsetLeft'+svgRef.offsetLeft+' svgRef.offsetTop'+svgRef.offsetTop);"
				// + "console.log('x end '+cursor.x+' y'+cursor.y);"
				+ "return cursor;"
				+ "};"
				+ "function showMass(evt,mz) {mzInfo.firstChild.nodeValue = 'mz = '+mz;mzInfo.setPosition(getEvtPosition(evt)); }"
				+ "function hideMass() {mzInfo.setPosition(cursorOut);}";
		javascriptElement.appendText(code);
		this.getSvgRoot().setOnLoad("initSvg()");
	}

	@Override
	protected void render() {
		super.render();
		ArrayList<SvgPeak> peakList = null;
		try {
			peakList = this.getSvgPeakList();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		for (SvgPeak peak : peakList) {
			// peak.printIonName();
			peak.setOnMouseOver("if (typeof showMass == 'function') {showMass(evt, '"
					+ peak.getMz() + "')};");
			// peak.setOnMouseOut("if (typeof hideMass == 'function') {hideMass()};");
		}

	}

	@SuppressWarnings("unused")
	private String writeGetPositionFunction() {
		String function = "function getPosition(element) {\n"
				+ "var left = 0;var top = 0;var el=element;\n"

				// + "console.log('e2 el.offsetLeft '+el.offsetLeft);"
				// + "console.log('e2 tag '+el.tagName);"
				+ "while( el && !isNaN( el.offsetLeft ) && !isNaN( el.offsetTop ) ) {"
				// + "console.log('e2 tag '+el.tagName);"
				+ "left += el.offsetLeft - el.scrollLeft;"
				+ "top += el.offsetTop - el.scrollTop;" + "el = el.parentNode;"
				+ "}" + "\nreturn {x:left, y:top};"

				+ "}";
		return function;
	}

	@SuppressWarnings("unused")
	private String writeGetScrollPositionFunction() {
		String function = "function getScrollPosition() {\n"
				+ "var cursorb = {x:0, y:0};"
				// + "if (typeof jQuery != 'undefined' ) {"
				// + "cursorb.x += $(window).scrollLeft();"
				// + "cursorb.y += $(window).scrollTop();} else {\n"
				+ "cursorb.x += document.documentElement.scrollLeft;"
				+ "cursorb.y += document.documentElement.scrollTop;"
				// +
				// "cursorb.x += (document.documentElement.scrollLeft ?document.documentElement.scrollLeft :document.body.scrollLeft);"
				// +
				// "cursorb.y += (document.documentElement.scrollTop ?document.documentElement.scrollTop : document.body.scrollTop);"
				// + "\n}"

				+ "return cursorb;" + "}";
		return function;
	}

}
