package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.filter_print.FilterGroupFromFastaDatabase;
import fr.inra.pappso.xtandempipeline.filter_print.filter_base;
import fr.inra.pappso.xtandempipeline.filter_print.print_base;
import fr.inra.pappso.xtandempipeline.filter_print.swt_table_base;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;

public class Identification {

	private static final Logger logger = Logger.getLogger(Identification.class);

	private ArrayList<Peptide> peptide_liste;

	private GroupSet grouping;

	private MatchList protein_liste;

	private ModifsList mods;

	private MsRunSet msRunSet = new MsRunSet();
	
	private DatabaseSet databaseSet = new DatabaseSet();

	public Identification() {
		this.peptide_liste = new ArrayList<Peptide>();
		this.protein_liste = new MatchList();
		this.grouping = null;
		this.mods = new ModifsList();
	}
	
	@Override
	protected void finalize() throws Throwable {
		logger.debug("finalize identification");
		logger.debug("Match : "+protein_liste.size());
		logger.debug("Peptide : "+peptide_liste.size());
	}
	
	public void addMsRun(String S) {
		msRunSet.getInstance(S);
	}

	public void add_peptide(Peptide P) {
		// if (!this.peptide_liste.contains(P))
		this.peptide_liste.add(P);
	}

	public ModifsList get_modifs_liste() {
		return (mods);
	}

	public void add_modifs_to_liste(Modifs mod) {
		this.mods.add_modifs(mod);
	}

	public boolean contain_protein(Protein P) {
		if (this.protein_liste.containsKey(P.get_cle_accession()))
			return (false);
		else {
			return (true);
		}
	}

	public void add_protein(Protein P, Match M) {
		this.protein_liste.put(P.get_cle_accession(), M);
	}

	public ArrayList<Peptide> get_peptide_liste() {
		return (this.peptide_liste);
	}

	public GroupSet getGrouping() {
		return grouping;
	}

	public MatchList get_protein_liste() {
		return (this.protein_liste);
	}

	public MsRunSet get_samples() {
		return (msRunSet);
	}

	public MsRunSet getMsRunSet() {
		return this.msRunSet;
	}
	
	public Match get_match(Protein P) {
		if (this.protein_liste.containsKey(P.get_cle_accession()))
			return (this.protein_liste.get(P.get_cle_accession()));
		else {
			return (null);
		}
	}

	public void filtered_match(filter_base filter) throws MSMSException {
		int valid = filter.execute(this.protein_liste);
		logger.info("Number of Valid Match " + valid);
	}

	public void print_result(print_base print) throws Exception {
		print.execute(grouping);
	}

	public void view_result(swt_table_base table) throws Exception {
		table.execute(grouping);
	}

	public void remove_contaminant_submatch(File database) {
		//TODO Remove group containing prot from this database
		new FilterGroupFromFastaDatabase(database).filter(grouping);
	}

	public void setProteinListValideToGroupeSet() {
		// Seul les match dans le groupSet sont valides
		logger.info("Unvalide match not present in Group");
		for (Match m : protein_liste.getMatchListes()) {
			m.setIsvalidate(false);
		}

		for (Group group : grouping.getGroupList()) {
			for (SubGroup sg : group.getSubGroupSet()) {
				for (HashSampleScanSetProt hk : sg
						.getHashSampleScanSetProtSet()) {
					hk.getMatch().setIsvalidate(true);
				}
			}
		}
	}

	public void performedGroupSet() throws Exception {
		logger.info("Performed grouping");
		GroupSet resultGroupSet = new GroupSet();

		SubGroupSet subGroupSet = new SubGroupSet();
		for (Match m : protein_liste.values()) {
			if (m.isIsvalidate()) {
				subGroupSet.addHashSampleScanSetProt(new HashSampleScanSetProt(
						m));
			}
		}
		logger.info("Number of subgroup before group creation : "+subGroupSet.size());
		
		resultGroupSet.groupingSubGroupSet(subGroupSet);
		grouping = resultGroupSet;
	}

	public void performedPhosphoGroupSet() throws Exception {
		GroupSet resultGroupSet = new GroupSet();

		SubGroupSet subGroupSet = new SubGroupSet();
		for (Match m : protein_liste.values()) {
			if (m.isIsvalidate()) {
				ArrayList<HashSampleScanSetPhospho> phosphoSites = m
						.getHashSampleScanSetPhosphoList();
				for (HashSampleScanSetPhospho phospho : phosphoSites) {
					subGroupSet.addHashSampleScanSetProt(phospho);
				}
			}
		}

		/*
		 * for (SubGroup sg:subGroupSet) { sg.loggerDebug(); }
		 */
		resultGroupSet.groupingSubGroupSet(subGroupSet);
		this.grouping = resultGroupSet;
	}

	public ArrayList<Database> get_database_list() {
		ArrayList<Database> datas = new ArrayList<Database>(databaseSet);
		Collections.sort(datas);
		return datas;
	}

	public DatabaseSet getDatabaseSet() {
		return databaseSet;
	}
}
