package fr.inra.pappso.xtandempipeline.grouping;

import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;

public class Group extends HashKey4GroupsSet implements Comparable<Group> {

	private static final long serialVersionUID = -917565136462863548L;

	private static final Logger logger = Logger.getLogger(Group.class);

	private SubGroupSet subGroupList = new SubGroupSet();

	private Integer number = null;

	public Group(SubGroup subGroup) throws Exception {
		subGroupList.add(subGroup);
		HashKey4GroupsSet peptideSet = subGroup.getHashKey4GroupsSet();
		this.addAll(peptideSet);
	}

	@Override
	public String toString() {
		String groupStr = new String();
		for (SubGroup sg : subGroupList) {
			groupStr = groupStr + sg.toString();
		}
		return groupStr;

	}

	public boolean fusionWith(Group incomingGroup) throws Exception {
		// fusion with the incoming group if this contains common elements
		// AND if removing non informative elements does not impact grouping
		boolean fusion = false;

		if (this.containsCommonHashKey4GroupsWith(incomingGroup)) {
			fusion = true;
			/*
			 * //logger.debug("no non informative keys removed, fusioning !!!!!")
			 * ;
			 * 
			 * // remove redundancy in the future group of both : self and
			 * incoming // group boolean changed =
			 * removeNonInformativeConsideringGroup(incomingGroup);
			 * 
			 * if (changed) { if
			 * (this.containsCommonHashKey4GroupsWith(incomingGroup)) { fusion =
			 * true;
			 * logger.debug("non informative keys removed, fusioning !!!!!"); }
			 * else { logger.debug(
			 * "non informative keys removed, but this cancel the fusion !!!!!"
			 * ); logger.debug("group : \n"+this.toString());
			 * logger.debug("incoming group : \n"+this.toString()); }
			 * 
			 * } else { fusion = true;
			 * //logger.debug("no non informative keys removed, fusioning !!!!!"
			 * ); }
			 */
		}

		if (fusion) {
			this.addAll(incomingGroup);
			subGroupList.addAll(incomingGroup.subGroupList);
		}
		return fusion;
	}

	private boolean containsCommonHashKey4GroupsWith(Group incomingGroup)
			throws Exception {
		boolean yes = false;
		for (HashKey4Groups hashPeptide : incomingGroup) {
			if (hashPeptide == null) {
				String message = "hashPeptide is null";
				logger.error(message);
				throw new Exception(message);
			}
			if (this.contains(hashPeptide)) {
				yes = true;
				break;
			} else {
				// logger.debug(" hashPeptide " + hashPeptide +
				// "("+hashPeptide.hashCode()+") is NOT in group ");
			}
		}
		return yes;
	}

	public int compareTo(Group o) {
		int i = o.getHashSampleScanSet().size()
				- this.getHashSampleScanSet().size();
		if (i != 0)
			return i;
		return (this.getHashSampleScanSet().compareTo(o.getHashSampleScanSet()));
	}

	public HashSampleScanSet getHashSampleScanSet() {
		HashSampleScanSet sampleScanSet = new HashSampleScanSet();
		for (SubGroup sg : this.subGroupList) {
			sampleScanSet.addAll(sg);
		}
		return sampleScanSet;
	}

	protected void setNumber(int i) {
		number = i;
	}

	public Integer getNumber() {
		return number;
	}

	public void loggerDebug() {
		logger.debug("group num : " + this.getNumber());
		for (SubGroup sg : this.subGroupList.getSubGroupList()) {
			logger.debug("sub group " + sg.getNumber());

			sg.loggerDebug();
		}
	}

	@SuppressWarnings("unused")
	private boolean removeNonInformativeConsideringGroup(Group incomingGroup)
			throws Exception {

		boolean changed = false;
		HashSampleScanSet sampleScanSetOverall = new HashSampleScanSet();
		sampleScanSetOverall.addAll(this.getHashSampleScanSet());
		sampleScanSetOverall.addAll(incomingGroup.getHashSampleScanSet());

		// remove in self
		SubGroupSet newSubGroupListToKeepSelf = new SubGroupSet();

		for (HashSampleScan SampleScan : sampleScanSetOverall) {
			int nbPresence = 0;
			SubGroup sgSpec = null;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(SampleScan)) {
					nbPresence++;
					sgSpec = sg;
				}
			}
			if (nbPresence == 1) {
				newSubGroupListToKeepSelf.add(sgSpec);
			}
		}

		if (this.subGroupList.size() > newSubGroupListToKeepSelf.size()) {
			logger.debug((this.subGroupList.size() - newSubGroupListToKeepSelf
					.size()) + " subgroups removed : non informative");
			this.subGroupList.clear();
			this.subGroupList.addAll(newSubGroupListToKeepSelf);

			HashKey4GroupsSet newKeys = new HashKey4GroupsSet();
			for (SubGroup sg : subGroupList) {
				newKeys.addAll(sg.getHashKey4GroupsSet());
			}
			if (this.size() > newKeys.size()) {
				logger.debug(" this group has changed : " + this.size() + " > "
						+ newKeys.size());
				this.clear();
				this.addAll(newKeys);
				changed = true;
				logger.debug(" this group new size : " + this.size());
			}
		}

		// remove in incoming

		SubGroupSet newSubGroupListToKeepIncoming = new SubGroupSet();

		for (HashSampleScan SampleScan : sampleScanSetOverall) {
			int nbPresence = 0;
			SubGroup sgSpec = null;
			for (SubGroup sg : incomingGroup.subGroupList) {
				if (sg.contains(SampleScan)) {
					nbPresence++;
					sgSpec = sg;
				}
			}
			if (nbPresence == 1) {
				newSubGroupListToKeepIncoming.add(sgSpec);
			}
		}

		if (incomingGroup.subGroupList.size() > newSubGroupListToKeepIncoming
				.size()) {
			logger.debug("incoming group before : \n"
					+ incomingGroup.toString());
			logger.debug((incomingGroup.subGroupList.size() - newSubGroupListToKeepIncoming
					.size())
					+ " subgroups removed from incomingGroup : non informative");
			incomingGroup.subGroupList.clear();
			incomingGroup.subGroupList.addAll(newSubGroupListToKeepIncoming);

			HashKey4GroupsSet newKeys = new HashKey4GroupsSet();
			for (SubGroup sg : incomingGroup.subGroupList) {
				newKeys.addAll(sg.getHashKey4GroupsSet());
			}
			if (incomingGroup.size() > newKeys.size()) {
				logger.debug(" incoming group has changed : "
						+ incomingGroup.size() + " > " + newKeys.size());
				incomingGroup.clear();
				incomingGroup.addAll(newKeys);
				changed = true;
				logger.debug(" incoming group new size : "
						+ incomingGroup.size());
				logger.debug("incoming group after : \n"
						+ incomingGroup.toString());
			}
		}
		return changed;
	}

	protected boolean removeNonInformative() throws Exception {
		boolean changed = false;
		HashSampleScanSet sampleScanSetOverall = this.getHashSampleScanSet();
		SubGroupSet newSubGroupListToKeep = new SubGroupSet();

		for (HashSampleScan SampleScan : sampleScanSetOverall) {
			int nbPresence = 0;
			SubGroup sgSpec = null;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(SampleScan)) {
					nbPresence++;
					sgSpec = sg;
				}
			}
			if (nbPresence == 1) {
				newSubGroupListToKeep.add(sgSpec);
			}
		}
		int deltaSize = this.subGroupList.size()- newSubGroupListToKeep.size();
		if (deltaSize == 0) {
			changed = false;
		}
		else if (deltaSize == 1) {
			logger.debug((this.subGroupList.size() - newSubGroupListToKeep
					.size()) + " subgroups removed : non informative");
			this.subGroupList.clear();
			this.subGroupList.addAll(newSubGroupListToKeep);
			changed = true;
		}
		else {
			//Plusieurs sous-groupe à enlever ???
			SubGroupSet sgListToRemove = new SubGroupSet();
			sgListToRemove.addAll(subGroupList);
			sgListToRemove.removeAll(newSubGroupListToKeep);
			
			SubGroup sgToRemove = null;
			int sgToRemoveSize = 1000000;
			
			for (SubGroup sg : sgListToRemove) {
				logger.debug("Remove non informative : "
						+ sg.getHashSampleScanSetProtList().get(0)
								.getMatch().get_protein_match()
								.get_cle_accession());
				if (sg.size() < sgToRemoveSize) {
					sgToRemove = sg;
					sgToRemoveSize = sg.size();
				}
			}
			this.subGroupList.remove(sgToRemove);
			this.removeNonInformative();
			
			changed=true;

		}
		return changed;
	}

	public SubGroupSet getSubGroupSet() {
		return this.subGroupList;
	}

	public int getNumberOfSpecificPeptideInSubGroup(
			HashSampleScanSetProt hashProt) {
		int count = 0;
		for (HashSampleScan hash : hashProt) {
			int view = 0;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(hash)) {
					view++;
				}
			}
			if (view == 1) {
				count++;
			}
		}
		return (count);
	}

	public int getNumberOfSpecificUniquePeptideInSubGroup(
			HashSampleScanSetProt hashProt) throws MSMSException {
		HashSet<String> sequences = new HashSet<String>();
		for (HashSampleScan hash : hashProt) {
			int view = 0;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(hash)) {
					view++;
				}
			}
			if (view == 1) {
				sequences.add(hashProt.getMatch()
						.getBestPeptideToHashSampleScan(hash).get_sequence());
			}
		}
		return (sequences.size());
	}

	public int getNumberOfSpecificPeptideInSubGroupToSample(
			HashSampleScanSetProt hashProt, MsRun sample) {
		int count = 0;
		for (HashSampleScan hash : hashProt) {
			if (hash.getMsRun().equals(sample)) {
				int view = 0;
				for (SubGroup sg : this.subGroupList) {
					if (sg.contains(hash)) {
						view++;
					}
				}
				if (view == 1) {
					count++;
				}
			}
		}
		return (count);
	}

	public SubGroupSet getSubGroupContaining(HashSampleScan hash) {
		SubGroupSet contain = new SubGroupSet();
		for (SubGroup sg : this.subGroupList) {
			if (sg.contains(hash)) {
				contain.add(sg);
			}
		}
		return (contain);
	}

	public HashSet<Group> reGroup() throws Exception {
		GroupSet newGroupSet = new GroupSet();
		for (SubGroup sg : this.subGroupList) {
			newGroupSet.addSubGroup(sg);
		}
		logger.debug("new size after reGroup : " + newGroupSet.size());

		return newGroupSet.getHashSetOfGroups();
	}
}
