package fr.inra.pappso.xtandempipeline.grouping;

import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.Peptide;

public class HashSampleScanSetProt extends HashSampleScanSet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7558622484317846088L;
	/**
	 * 
	 */
	private static final Logger logger = Logger
			.getLogger(HashSampleScanSetProt.class);

	protected Match match;

	public HashSampleScanSetProt(Match match) throws MSMSException {
		this.setMatch(match);
		for (Peptide peptide : match.get_valid_peptides()) {
			HashSampleScan hashSample = peptide.getSampleScan();
			if (hashSample == null) {
				String message = "hashSample is null";
				logger.error(message);
			}
			this.add(hashSample);
		}
	}

	@Override
	public void clear() {
		match = null;
		super.clear();
	}

	@Override
	public String toString() {
		return match.get_protein_match().get_cle_accession();
	}

	protected HashSampleScanSetProt() {
		match = null;
	}

	public int compareTo(HashSampleScanSet arg0) {
		// TODO Auto-generated method stub
		int i = this.size() - arg0.size();
		if (i != 0)
			return i;
		return this
				.getMatch()
				.get_protein_match()
				.compareTo(
						((HashSampleScanSetProt) arg0).getMatch()
								.get_protein_match());
	}

	@Override
	public boolean equals(Object toCompare) {
		if (super.equals(toCompare)) {
			HashSampleScanSetProt incoming = (HashSampleScanSetProt) toCompare;
			if (incoming.getMatch().get_protein_match().get_cle_accession()
					.equals(this.match.get_protein_match().get_cle_accession())) {
				return(true);
			}
			return(false);
		}
		return false;
	}

	protected void setMatch(Match match) throws MSMSException {
		if (match == null) {
			String message = "Macth null";
			logger.error(message);
			throw new MSMSException(message);
		}
		this.match = match;
	}

	public Match getMatch() {

		return match;
	}

	public HashKey4GroupsSet getHashKey4GroupsSet() throws Exception {
		HashPeptideSet hashPeptideSet = new HashPeptideSet();

		for (Peptide peptide : match.get_valid_peptides()) {
			HashPeptide p = peptide.getHashPeptide();
			if (p == null) {
				String message = "hashPeptide is null";
				logger.error(message);
				throw new Exception(message);
			}

			hashPeptideSet.add(p);
		}

		return hashPeptideSet;
	}

	public int getUniquePeptideHashSampleScan() throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (HashSampleScan hash : this) {
			Peptide pep = this.match.getBestPeptideToHashSampleScan(hash);
			sequence.add(pep.get_sequence());
		}
		return (sequence.size());
	}

	public int getTotalHashSampleScan() {
		return (this.size());
	}

	public int getTotalHashSampleScantoMsRun(MsRun sample) {
		int count = 0;
		for (HashSampleScan hash : this) {
			if (hash.getMsRun().equals(sample))
				count++;
		}
		return (count);
	}

	public int getUniqueHashPeptideSampleScantoMsRun(MsRun sample) throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (HashSampleScan hash : this) {
			Peptide pep = this.match.getBestPeptideToHashSampleScan(hash);
			if (pep.getMsRun().equals(sample)) {
				sequence.add(pep.get_sequence());
			}
		}
		return (sequence.size());
	}
}
