package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.w3c.dom.*;

import javax.xml.parsers.*;
import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.*;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;


public class MassChroQML {
	private static final Logger logger = Logger.getLogger(MassChroQML.class);
	
	private Identification ident;

	private ModifsList mods;

	private File qmscpp_file;

	private Document doc;

	private Element racine;

	private Element groups;

	private Element rawdata;

	private Element proteins;

	private Element peptide_list;

	private Element alignements;

	private Element isotopes;

	private Element quantification_methods;
	
	private Element quantification;
	
	private String groupe_id;

	private Hashtable<String, String> prot_to_id;

	private Hashtable<String, Element> seq_to_element;

	private Hashtable<String, String> sample_to_id;

	public MassChroQML(File F) throws Exception {
		logger.info("Save result in MassChroQ file");
		this.qmscpp_file = F;
		this.prot_to_id = new Hashtable<String, String>();
		this.seq_to_element = new Hashtable<String, Element>();
		this.sample_to_id = new Hashtable<String, String>();
		this.groupe_id = "G1";
		this.mods = new ModifsList();

		try {
			// creation et ouverture d'un DOM
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();
			this.doc = parser.newDocument();
			this.racine = doc.createElement("masschroq");
			doc.appendChild(this.racine);

		} catch (Exception e) {
			throw (e);
		}
	}

	private void initialise() throws Exception {
		// ajout des différentes racines du fichier
		rawdata = doc.createElement("rawdata");
		racine.appendChild(rawdata);

		groups = doc.createElement("groups");
		racine.appendChild(groups);

		proteins = doc.createElement("protein_list");
		racine.appendChild(proteins);

		peptide_list = doc.createElement("peptide_list");
		racine.appendChild(peptide_list);

		if (mods.get_modifs_order().length > 0) {
			isotopes = doc.createElement("isotope_label_list");
			racine.appendChild(isotopes);
		}

		alignements = doc.createElement("alignments");
		racine.appendChild(alignements);

		quantification_methods = doc.createElement("quantification_methods");
		racine.appendChild(quantification_methods);
		
		quantification = doc.createElement("quantification");
		racine.appendChild(quantification);

		write();

	}

	public void set_identification(Identification id) throws Exception {
		try {
			if (this.ident != null)
				throw new MSMSException(
						"Adding more than one identification result is not implemented");

			this.ident = id;

			// on ajoute les balises
			initialise();

			// ajout des echantillons_msrun
			this.add_rawdata_groups();
			// sauvegarde
			this.write();

			// ajout des proteines
			this.add_proteins();
			// sauvegarde
			this.write();

			// ajout des peptides
			this.add_peptides();
			// sauvegarde
			this.write();

			// ajout des modifications isotopics
			this.add_isotopes();
			// sauvegarde
			this.write();

			// ajout des methodes
			this.add_alignement();
			// sauvegarde
			this.write();

			// ajout des methodes
			this.add_methods();
			// sauvegarde
			this.write();
			
			//ajout des instructions de quanti
			this.add_quantify();
			// sauvegarde
			this.write();

		} catch (Exception e) {
			throw (e);
		}

	}

	private void add_isotopes() {
		Modifs[] mds = mods.get_modifs_order();
		for (int i = 0; i < mds.length; i++) {
			Element isotope_label = doc.createElement("isotope_label");
			isotope_label.setAttribute("id", "iso" + (i + 1));
			Element mod = doc.createElement("mod");
			mod.setAttribute("value", "" + mds[i].get_modvalue());
			mod.setAttribute("at", "Nter,Cter,A,K,L...");
			isotope_label.appendChild(mod);
			isotopes.appendChild(isotope_label);
		}
	}

	private void add_alignement() {
		// methods
		Element alignment_methods = doc.createElement("alignment_methods");
		alignements.appendChild(alignment_methods);
		
		Element alignment_method = doc.createElement("alignment_method");
		alignment_methods.appendChild(alignment_method);

		alignment_method.setAttribute("id", "ms2_1");
		Element ms2 = doc.createElement("ms2");
		Element tendency = doc.createElement("ms2_tendency_halfwindow");
		tendency.setTextContent("10");
		ms2.appendChild(tendency);
		Element smoothing = doc.createElement("ms2_smoothing_halfwindow");
		smoothing.setTextContent("15");
		ms2.appendChild(smoothing);
		Element smoothing2 = doc.createElement("ms1_smoothing_halfwindow");
		smoothing2.setTextContent("0");
		ms2.appendChild(smoothing2);
		alignment_method.appendChild(ms2);

		// groupe d'alignement
		Element align = doc.createElement("align");
		align.setAttribute("group_id", this.groupe_id);
		align.setAttribute("method_id", "ms2_1");
		align.setAttribute("reference_data_id", "samp0");
//		align.setAttribute("method_id", "obiwarp1");
		this.alignements.appendChild(align);

	}

	private void add_methods() {
		Element quantification_method = doc.createElement("quantification_method");
		quantification_method.setAttribute("id", "quant1");
		quantification_methods.appendChild(quantification_method);
		
		Element xic_extraction = doc.createElement("xic_extraction");
		xic_extraction.setAttribute("xic_type", "max");
		quantification_method.appendChild(xic_extraction);
		Comment type = doc.createComment("max : XIC on BasePeak; sum : XIC on TIC");
		quantification_method.appendChild(type);
		
		// selection
		Element ppm_range = doc.createElement("ppm_range");
		ppm_range.setAttribute("min", "10");
		ppm_range.setAttribute("max", "10");
		xic_extraction.appendChild(ppm_range);
		Comment mz_range = doc.createComment("For XIC extraction on Da use: mz_range");
//		mz_range.setAttribute("min", "0.5");
//		mz_range.setAttribute("max", "1.5");
		xic_extraction.appendChild(mz_range);
		
		// xic_filter
		Element xic_filters = doc.createElement("xic_filters");
		quantification_method.appendChild(xic_filters);

		Element anti_spike = doc.createElement("anti_spike");
		anti_spike.setAttribute("half", "5");
		xic_filters.appendChild(anti_spike);

		Element background = doc.createElement("background");
		background.setAttribute("half_mediane", "5");
		background.setAttribute("half_min_max", "20");
		xic_filters.appendChild(background);

		// detection michel trop bien
		Element peak_detection = doc.createElement("peak_detection");
		quantification_method.appendChild(peak_detection);
		
		Element detection = doc.createElement("detection_zivy");
		Element smoothing = doc.createElement("mean_filter_half_edge");
		smoothing.setTextContent("1");
		detection.appendChild(smoothing);
		Element minmax = doc.createElement("minmax_half_edge");
		minmax.setTextContent("3");
		detection.appendChild(minmax);
		Element maxmin = doc.createElement("maxmin_half_edge");
		maxmin.setTextContent("2");
		detection.appendChild(maxmin);
		Element tic_start = doc.createElement("detection_threshold_on_max");
		tic_start.setTextContent("5000");
		detection.appendChild(tic_start);
		Element tic_stop = doc.createElement("detection_threshold_on_min");
		tic_stop.setTextContent("3000");
		detection.appendChild(tic_stop);
		peak_detection.appendChild(detection);

	}
	
	private void add_quantify(){
		Element result =  doc.createElement("quantification_results");
		quantification.appendChild(result);
		Element output = doc.createElement("quantification_result");
		output.setAttribute("output_file", "XIC_result");
		output.setAttribute("format", "tsv");
		result.appendChild(output);
		
		Element trace = doc.createElement("quantification_traces");
		quantification.appendChild(trace);
		Element peptides_traces = doc.createElement("peptide_traces");
		peptides_traces.setAttribute("peptide_ids", "pep0 pep1");
		peptides_traces.setAttribute("output_dir", "peplist_xics_traces/");
		peptides_traces.setAttribute("format", "tsv");
		trace.appendChild(peptides_traces);
		
		Element quantify = doc.createElement("quantify");
		quantify.setAttribute("id", "q1");
		quantify.setAttribute("withingroup", this.groupe_id);
		quantify.setAttribute("quantification_method_id", "quant1");
		Element peptides_in_peptide_list = doc
				.createElement("peptides_in_peptide_list");
		peptides_in_peptide_list.setAttribute("mode", "real_or_mean");

		// ajout des labels isotopics dans la methode
		if (mods.size() > 0) {
			String label = "";
			for (int z = 0; z < mods.size(); z++) {
				label = label + " " + "iso" + (z + 1);
			}
			peptides_in_peptide_list.setAttribute("isotope_label_refs", label.trim());
		}
		quantify.appendChild(peptides_in_peptide_list);
		quantification.appendChild(quantify);

	}
	

	private void add_rawdata_groups() {
		for(MsRun sample : ident.get_samples().getMsRunList()){
			String id_samp = "samp" + this.sample_to_id.size();
			String name = sample.getSampleName();
			// ajout dans les hashtable
			sample_to_id.put(name, id_samp);
			//name = name.replaceAll(".xml", ".mzXML");
			name = name+".mzXML";
			
			// balise rawdata
			Element data_file = doc.createElement("data_file");
			data_file.setAttribute("id", id_samp);
			data_file.setAttribute("type", "profile");
			data_file.setAttribute("format", "mzxml");
			data_file.setAttribute("path", "" + name);
			rawdata.appendChild(data_file);
		}

		// //balise groups
		Element group = doc.createElement("group");
		group.setAttribute("id", this.groupe_id);
		String raw_list="";
		for(MsRun sample : ident.get_samples().getMsRunList()){
			raw_list = raw_list + " " + sample_to_id.get(sample.getSampleName());
		}
		group.setAttribute("data_ids", raw_list.trim());
		groups.appendChild(group);
	}

	private void add_proteins() {
		for(Group group : ident.getGrouping().getGroupList()){
			for(SubGroup sg : group.getSubGroupSet().getSubGroupList()){
				Protein top = sg.getHashSampleScanSetProtList().get(0).getMatch().get_protein_match();
				
				// balise proteines
				String prot_id = this.getSubGroupGroupNumber(group, sg);

				Element protein = doc.createElement("protein");
				protein.setAttribute("id", prot_id);
				protein.setAttribute("desc", top.get_description());
				proteins.appendChild(protein);
				this.prot_to_id.put(top.get_cle_accession(), prot_id);
			}
		}
	}

	private void add_peptides() throws Exception {
		Hashtable<String, HashSet<String>> pep_sequence_to_prot = new Hashtable<String, HashSet<String>>();
		for(Group group : ident.getGrouping().getGroupList()){
			for(HashSampleScan hash : group.getHashSampleScanSet().getHashSampleScanList()){
				SubGroupSet set = group.getSubGroupContaining(hash);
				ArrayList<SubGroup> sgList = set.getSubGroupList();
				Match top = sgList.get(0).getHashSampleScanSetProtList().get(0).getMatch();
				Peptide pep = top.getBestPeptideToHashSampleScan(hash);
				
				Element peptide;
				String seqMod = pep.get_LI_sequenceModifsWithoutMs3();
				if (seq_to_element.containsKey(seqMod)) {
					peptide = this.seq_to_element.get(seqMod);
				} else {
					peptide = doc.createElement("peptide");
					String pep_id = "pep" + seq_to_element.size();
					peptide.setAttribute("id", pep_id);
					this.seq_to_element.put(seqMod,
							peptide);
					peptide.setAttribute("seq", pep.get_sequence());
					peptide.setAttribute("mh", ""
							+ pep.get_mh_theo_without_isotope(true));
					peptide.setAttribute("mods", ""
							+ pep.get_Modifs_formater_without_isotope());
					this.peptide_list.appendChild(peptide);
					// table temporaire pour prot_id
					HashSet<String> temp = new HashSet<String>();
					//Hashtable<String, String> temp = new Hashtable<String, String>();
					pep_sequence_to_prot.put(seqMod,temp);
				}
				//On ajoute une observation
				Element observed = doc.createElement("observed_in");
				// eda choisir mode export
				// observed.setAttribute("rt", "" + peps[j].get_RT_in_sec());
				observed.setAttribute("scan", "" + pep.get_scan());
				observed.setAttribute("data", this.sample_to_id.get(pep
						.get_sample()));
				observed.setAttribute("z", "" + pep.get_charge());
				peptide.appendChild(observed);
				
				for(SubGroup sgTemp : sgList){
					HashSet<String> temp = pep_sequence_to_prot.get(seqMod);
					temp.add(this.getSubGroupGroupNumber(group, sgTemp));
				}
				
				
			}

			// on reparcourt la liste pour ajouter les prots id
			for(String seq : pep_sequence_to_prot.keySet()){
				Element peptide = this.seq_to_element.get(seq);
				HashSet<String> temp = pep_sequence_to_prot.get(seq);
				Iterator<String> prots = temp.iterator();
				String ids = prots.next();
				while (prots.hasNext())
					ids = ids + " " + prots.next();
				peptide.setAttribute("prot_ids", ids.trim());
			}

		}
	}

	private void write() throws Exception {
		// écriture d'un document
		try {
			TransformerFactory tfact = TransformerFactory.newInstance();
			Transformer transformer = tfact.newTransformer();
			transformer.setOutputProperty("encoding", "UTF-8");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			DOMSource source = new DOMSource(this.doc);
			PrintWriter fw = new PrintWriter(this.qmscpp_file, "UTF-8");
			StreamResult result = new StreamResult(fw);
			transformer.transform(source, result);
		} catch (Exception e) {
			throw new MSMSException(
					"Immpossible to save/write file : "
							+ this.qmscpp_file.getName());
		}
	}

	public void set_isotopes_mods(ModifsList mods) {
		this.mods = mods;
	}
	
	private String getSubGroupGroupNumber(Group group,SubGroup sg){
		String prot_id = "P"+group.getNumber()+ ".";
		if(sg.getNumber()<10)
			prot_id += "0"+sg.getNumber();
		else
			prot_id += sg.getNumber();
		return(prot_id);
	}

}
