package fr.inra.pappso.xtandempipeline.sax_parse;

import java.io.File;
import java.net.URL;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.Peptide;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;
import fr.inra.pappso.xtandempipeline.thread.process;

public class load_project extends org.xml.sax.helpers.DefaultHandler {
	private static final Logger logger = Logger.getLogger(load_project.class);

	private Protein prot_;

	private Peptide pep_;

	private Identification identification_;

	// private ArrayList<Identification> identifications;

	private Match match_;

	private String Last_element_;

	private String Start_element_;

	private String C_data;

	private DataType_Base datatype;

	private process pro;

	private String version;

	public load_project(config conf, process p) {
		super();
		// identifications =
		// XtandemPipelineSession.getInstance().getIdentificationList();
		datatype = XtandemPipelineSession.getInstance().getDataTypeBase();
		this.pro = p;
	}

	public void error(SAXParseException e) throws SAXParseException {
		throw e;
	}

	public void startDocument() throws SAXException {
		identification_ = new Identification();
		prot_ = null;
		pep_ = new Peptide();
		match_ = new Match();
		Last_element_ = "";
		Start_element_ = "";
		C_data = "";
		version = null;
		logger.info("Load project ...");
	}

	public void endDocument() throws SAXException {
		logger.info("Load project finish");
	}

	public void startElement(String namespaceURI, String simpleName,
			String qualifiedName, Attributes attrs) throws SAXException {

		try {
			if (pro.isStop()) {
				throw new SAXException("sax parser stopped");
			}

			Start_element_ = simpleName;
			if (Start_element_.equals(""))
				Start_element_ = qualifiedName;
			// System.out.println("Start= "+Start_element_);
			if (Start_element_.equals("xtandem_pipeline")) {
				logger.debug("xtandem_pipeline");
				this.version = attrs.getValue("version");
			}
			if (Start_element_.equals("PROTICport")) {
				this.startElementPROTICport(namespaceURI, attrs);
			}
			if (Start_element_.equals("information")) {
				logger.debug("information");
				datatype = datatype.getType(attrs.getValue("Data_Type"));
				XtandemPipelineSession.getInstance().setDataTypeBase(datatype);
				this.pro.set_max_progress(Integer.parseInt(attrs
						.getValue("match_number")));
			}
			if (Start_element_.equals("filter_params")) {
				logger.debug("filter_params");
				datatype.getConf().set_peptide_evalue(
						Float.valueOf(attrs.getValue("pep_evalue"))
								.floatValue());
				datatype.getConf().set_protein_evalue(
						Float.valueOf(attrs.getValue("prot_evalue"))
								.floatValue());
				datatype.getConf().set_peptide_number(
						Integer.parseInt(attrs.getValue("pep_number")));
				datatype.getConf().set_protein_filter_to_complete(
						attrs.getValue("filter_to_all").equals("true"));
				datatype.getConf().set_database_filter(
						new File(attrs.getValue("database_filter")));
			}

			if (Start_element_.equals("identification")) {
				logger.debug("identification");
				identification_ = new Identification();
			}
			if (Start_element_.equals("sample")) {
				logger.debug("sample");
				identification_.addMsRun(attrs.getValue("value"));
			}
			if (Start_element_.equals("match")) {
				logger.debug("match");
				match_ = new Match();
				match_.setIsvalidate(attrs.getValue("validate").equals("true"));
			}

			if (Start_element_.equals("protein")) {
				logger.debug("protein");
				this.startElementprotein(namespaceURI, attrs);
			}
			if (Start_element_.equals("protein_evalue")) {
				logger.debug("protein_evalue");
				prot_.set_evalue(
						identification_.getMsRunSet().getInstance(
								attrs.getValue("sample")),
						Float.valueOf(attrs.getValue("evalue")).floatValue());
			}
			if (Start_element_.equals("peptide")) {
				logger.debug("peptide");
				this.startElementpeptide(namespaceURI, attrs);
			}
			if (Start_element_.equals("modif")) {
				logger.debug("modif");
				this.startElementmodif(namespaceURI, attrs);
			}
			if (Start_element_.equals("modifs_mass")) {
				logger.debug("modifs_mass");
				float modvalue = Float.valueOf(attrs.getValue("modvalue"))
						.floatValue();
				identification_.add_modifs_to_liste(new Modifs(modvalue));
			}
		} catch (MSMSException e) {
			String message = "ERROR reading xpip project file :\n"
					+ e.getMessage();
			logger.error(message);
			throw new SAXException(message);
		}

		C_data = "";
	}

	private void startElementPROTICport(String namespaceURI, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementPROTICport begin");
		try {
			if (attrs.getValue("url") == null) {
				throw new MSMSException("url is null");
			}
			URL url = new URL(attrs.getValue("url"));
			String login = attrs.getValue("user");
			String password = attrs.getValue("password");
			XtandemPipelineSession.getInstance().setPROTICportDataSource();
			XtandemPipelineSession.getInstance().setPROTICportController(url,
					login, password);
		} catch (Exception e) {
			String message = "ERROR reading PROTICport tag :\n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void startElementmodif(String namespaceURI, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementmodif begin");
		try {
			String aa = attrs.getValue("aa");
			if (aa == null) {
				throw new MSMSException("aa is null");
			}
			if (attrs.getValue("modvalue") == null) {
				throw new MSMSException("modvalue is null");
			}
			float modvalue = Float.valueOf(attrs.getValue("modvalue"))
					.floatValue();
			if (attrs.getValue("posi_in_prot") == null) {
				throw new MSMSException("posi_in_prot is null");
			}
			int posi_in_prot = Integer.parseInt(attrs.getValue("posi_in_prot"));
			pep_.set_Modifs(aa, posi_in_prot, modvalue);
		} catch (Exception e) {
			String message = "ERROR reading modif tag :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void startElementpeptide(String namespaceURI, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementpeptide begin");
		try {

			pep_ = new Peptide();
			// ajouter les elements du peptides
			if (attrs.getValue("sample_file") == null) {
				throw new MSMSException("sample_file is null");
			}
			IdentificationDataSource sample_file = XtandemPipelineSession
					.getInstance().getIdentificationDataSourceInstance(
							attrs.getValue("sample_file"));
			pep_.set_sample_id(
					this.identification_.getMsRunSet().getInstance(
							sample_file.getName()), sample_file);
			if (attrs.getValue("scan_in_xtandem") == null) {
				throw new MSMSException("scan_in_xtandem is null");
			}
			pep_.set_scan(Integer.parseInt(attrs.getValue("scan_in_xtandem")));
			if (attrs.getValue("scan") == null) {
				throw new MSMSException("scan is null");
			}
			pep_.set_scan(Integer.parseInt(attrs.getValue("scan")));
			if (attrs.getValue("RT") == null) {
				// throw new MSMSException("RT is null");
				logger.warn("RT is null");
			} else {
				pep_.set_RT(attrs.getValue("RT"));
			}
			if (attrs.getValue("mhplus_obser") == null) {
				throw new MSMSException("mhplus_obser is null");
			}
			pep_.set_mhplus_obser(Float.valueOf(attrs.getValue("mhplus_obser"))
					.floatValue());
			if (attrs.getValue("mhplus_theo") == null) {
				throw new MSMSException("mhplus_theo is null");
			}
			pep_.set_mhplus_theo(Float.valueOf(attrs.getValue("mhplus_theo"))
					.floatValue());
			if (attrs.getValue("deltamass") == null) {
				throw new MSMSException("deltamass is null");
			}
			pep_.set_deltamass(Float.valueOf(attrs.getValue("deltamass"))
					.floatValue());
			if (attrs.getValue("sequence") == null) {
				throw new MSMSException("sequence is null");
			}
			pep_.set_sequence(attrs.getValue("sequence"));
			if (attrs.getValue("pre") == null) {
				throw new MSMSException("pre is null");
			}
			pep_.set_pre(attrs.getValue("pre"));
			if (attrs.getValue("post") == null) {
				throw new MSMSException("post is null");
			}
			pep_.set_post(attrs.getValue("post"));
			if (attrs.getValue("start") == null) {
				throw new MSMSException("start is null");
			}
			pep_.set_start(Integer.parseInt(attrs.getValue("start")));
			if (attrs.getValue("stop") == null) {
				throw new MSMSException("stop is null");
			}
			pep_.set_stop(Integer.parseInt(attrs.getValue("stop")));
			if (attrs.getValue("charge") == null) {
				throw new MSMSException("charge is null");
			}
			pep_.set_charge(Integer.parseInt(attrs.getValue("charge")));
			if (attrs.getValue("evalue") == null) {
				throw new MSMSException("evalue is null");
			}
			pep_.set_evalue(Float.valueOf(attrs.getValue("evalue"))
					.floatValue());
			if (attrs.getValue("hypercorr") == null) {
				throw new MSMSException("hypercorr is null");
			}
			pep_.set_hypercorr(Float.valueOf(attrs.getValue("hypercorr"))
					.floatValue());
			if (attrs.getValue("validate") == null) {
				throw new MSMSException("validate is null");
			}
			pep_.setValidate(attrs.getValue("validate").equals("true"));
			// ajouter le peptide au match
			match_.add_peptide_match(pep_);
			identification_.add_peptide(pep_);
		} catch (Exception e) {
			String message = "ERROR reading peptide tag :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void startElementprotein(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementprotein begin");
		try {
			if (prot_ != null) {
				throw new MSMSException(
						"more than one protein in this match element");
			}
			prot_ = new Protein();
			// ajouter les elements de la prot
			if (attrs.getValue("description") == null) {
				throw new MSMSException("description is null");
			}
			prot_.set_description(attrs.getValue("description"));
			if (attrs.getValue("evalue") == null) {
				throw new MSMSException("evalue is null");
			}
			prot_.set_evalue(Float.valueOf(attrs.getValue("evalue"))
					.floatValue());
			if (attrs.getValue("URL") == null) {
				throw new MSMSException("URL is null");
			}
			prot_.setDatabase(identification_.getDatabaseSet().getInstance(
					attrs.getValue("URL")));
			if (attrs.getValue("peptide_number") == null) {
				throw new MSMSException("peptide_number is null");
			}
			prot_.set_peptide_number(Integer.parseInt(attrs
					.getValue("peptide_number")));
			logger.debug("startElementprotein end");
		} catch (Exception e) {
			String message = "ERROR reading protein tag :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void endElement(String namespaceURI, String simpleName,
			String qualifiedName) throws SAXException {
		Last_element_ = simpleName;
		if (Last_element_.equals(""))
			Last_element_ = qualifiedName;

		if (Last_element_.equals("protein")) {
			match_.set_protein_match(prot_);
		}
		if (Last_element_.equals("sequence")) {
			prot_.set_sequence(C_data);
		}
		if (Last_element_.equals("match")) {
			identification_.add_protein(prot_, match_);
			this.pro.add_current_progress();
			prot_ = null;
		}
		if (Last_element_.equals("identification")) {
			XtandemPipelineSession.getInstance().getIdentificationList()
					.add(identification_);
		}
		C_data = "";
	}

	public void characters(char buf[], int offset, int len) throws SAXException {
		C_data = C_data + new String(buf, offset, len);
		// System.out.println(C_data);
	}

	public String getVersion() {
		return version;
	}
}
