package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.xml.sax.*;
import java.util.Hashtable;
import java.util.Vector;

import fr.inra.pappso.xtandempipeline.class_msms.*;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;

public class xtandem extends org.xml.sax.helpers.DefaultHandler {
	private static final Logger logger = Logger.getLogger(xtandem.class);

	private IdentificationDataSource sample_;

	private Protein prot_;

	private Peptide pep_;

	private Identification identification_;

	private Hashtable<String, String> correction_scan;

	private Hashtable<String, String> correction_time;

	private Vector<Peptide> peptide_list;

	private Match match_;

	private String Last_element_;

	private String Start_element_;

	private String C_data;

	private String _RT;

	private int _scan;

	private float _mhplus_obser;

	private String _evalue_prot;

	private int _charge;

	private boolean prot_description;

	private boolean prot_sequence;

	private boolean spectre_number;

	public xtandem(Identification L, IdentificationDataSource S) {
		super();
		if (L == null) {
			identification_ = new Identification();
		} else {
			identification_ = L;
		}
		sample_ = S;
	}

	// Les methodes qui suivent sont appelées
	// automatiquement par l'analyseur
	// lorsqu'un événement est détecté
	// dans le fichier XML.
	public void error(SAXParseException e) throws SAXParseException {
		throw e;
	}

	public void startDocument() throws SAXException {
		prot_ = new Protein();
		pep_ = new Peptide();
		match_ = new Match();
		Last_element_ = "";
		Start_element_ = "";
		C_data = "";
		_scan = 0;
		_mhplus_obser = 0;
		_charge = 0;
		_evalue_prot = "";
		_RT = "";
		prot_description = false;
		spectre_number = false;
		prot_sequence = false;
		identification_.addMsRun(sample_.getName());
		correction_scan = new Hashtable<String, String>();
		correction_time = new Hashtable<String, String>();
		peptide_list = new Vector<Peptide>();
		logger.info("Parse Xtandem file in normal Mode : " + sample_.getName());
	}

	public void endDocument() throws SAXException {
		// correction du numéro de scan
		// Enumeration<match> E =
		// identification_.get_protein_liste().elements();
		// while (E.hasMoreElements()) {
		// peptide[] peps = E.nextElement().get_peptide_order_inclus_redon();
		// for (int i = 0; i < peps.length; i++) {
		for (int i = 0; i < peptide_list.size(); i++) {
			Peptide pep = peptide_list.elementAt(i);
			// if (pep.get_sample().equals(sample_.getName())) {
			if (correction_scan.containsKey("" + pep.get_scan()))
				pep.set_scan(Integer.parseInt(correction_scan.get(""
						+ pep.get_scan())));
			if (correction_time.containsKey("" + pep.get_scan()))
				pep.set_RT(correction_time.get("" + pep.get_scan()));
			// }
		}
		logger.info("Parse Xtandem file finish");
	}

	public void startElement(String namespaceURI, String simpleName,
			String qualifiedName, Attributes attrs) throws SAXException {
		Start_element_ = simpleName;
		if (Start_element_.equals(""))
			Start_element_ = qualifiedName;
		// System.out.println("Start= "+Start_element_);

		if (Start_element_.equals("group")) {
			if (attrs.getValue("type").equals("model")) {
				_scan = Integer.parseInt(attrs.getValue("id"));
				_mhplus_obser = Float.valueOf(attrs.getValue("mh"))
						.floatValue();
				_charge = Integer.parseInt(attrs.getValue("z"));
				_RT = attrs.getValue("rt");
			}
		}
		if (Start_element_.equals("protein")) {
			_evalue_prot = attrs.getValue("expect");
		}
		if (Start_element_.equals("note")) {
			if (attrs.getValue("label").equals("description"))
				prot_description = true;
			if (attrs.getValue("label").equals("Description"))
				spectre_number = true;
			// Ajouter la récupération du scan
		}

		if (Start_element_.equals("file")) {
			if (attrs.getValue("type").equals("peptide"))
				prot_.setDatabase(identification_.getDatabaseSet().getInstance(
						attrs.getValue("URL")));
		}

		if (prot_sequence) {
			if (prot_.have_a_sequence() == false)
				prot_.set_sequence(C_data);
			prot_sequence = false;
		}
		if (Start_element_.equals("peptide"))
			prot_sequence = true;

		if (Start_element_.equals("domain")) {
			// valeur généric du scan
			pep_.set_sample_id(
					identification_.getMsRunSet()
							.getInstance(sample_.getName()), sample_);
			pep_.set_scan(_scan);
			pep_.set_mhplus_obser(_mhplus_obser);
			pep_.set_charge(_charge);
			pep_.set_RT(_RT);

			// valeur specifique de la séquence matche
			pep_.set_start(Integer.parseInt(attrs.getValue("start")));
			pep_.set_stop(Integer.parseInt(attrs.getValue("end")));
			pep_.set_evalue(Float.valueOf(attrs.getValue("expect"))
					.floatValue());
			pep_.set_mhplus_theo(Float.valueOf(attrs.getValue("mh"))
					.floatValue());
			pep_.set_deltamass(Float.valueOf(attrs.getValue("delta"))
					.floatValue());
			pep_.set_hypercorr(Float.valueOf(attrs.getValue("hyperscore"))
					.floatValue());
			pep_.set_pre(attrs.getValue("pre"));
			pep_.set_post(attrs.getValue("post"));
			pep_.set_sequence(attrs.getValue("seq"));
		}
		if (Start_element_.equals("aa")) {
			pep_.set_Modifs(attrs.getValue("type"),
					Integer.parseInt(attrs.getValue("at")),
					Float.valueOf(attrs.getValue("modified")).floatValue());
			// modifs liste for isotopic analysis
			identification_.add_modifs_to_liste(new Modifs(Float.valueOf(
					attrs.getValue("modified")).floatValue()));
		}
		C_data = "";
	}

	public void endElement(String namespaceURI, String simpleName,
			String qualifiedName) throws SAXException {
		Last_element_ = simpleName;
		if (Last_element_.equals(""))
			Last_element_ = qualifiedName;

		if (Last_element_.equals("note")) {
			if (prot_description) {
				prot_.set_description(C_data);
				prot_description = false;
				if (identification_.contain_protein(prot_)) {
					match_.set_protein_match(prot_);
					identification_.add_protein(prot_, match_);
				}

				else {
					match_ = (Match) identification_.get_match(prot_);
					prot_ = match_.get_protein_match();
				}
				prot_.set_evalue(
						identification_.getMsRunSet().getInstance(
								sample_.getName()), Float.valueOf(_evalue_prot)
								.floatValue());
			}
			if (spectre_number) {
				String tmp = C_data.replaceAll(" ", "");
				String[] split = tmp.split("-");
				String num = split[0].replaceAll("Spectre", "");
				// System.out.println(num);
				try {
					int scan = Integer.parseInt(num);
					correction_scan.put("" + _scan, "" + scan);
					// System.out.println("" + _scan + " = " + scan);
					if (split.length > 1) {
						split[1] = split[1].replaceAll("RT", "");
						float rtime = Float.valueOf(split[1]).floatValue();
						correction_time.put("" + scan, "" + rtime);
						System.out.println("" + scan + " = " + rtime);
					}
				} catch (Exception e) {
				} finally {
					spectre_number = false;
				}
			}
		}

		if (Last_element_.equals("protein")) {
			match_ = new Match();
			prot_ = new Protein();
		}

		if (Last_element_.equals("domain")) {
			identification_.add_peptide(pep_);
			match_.add_peptide_match(pep_);
			peptide_list.add(pep_);
			prot_.add_one_peptide();
			pep_ = new Peptide();

		}

		C_data = "";
	}

	public void characters(char buf[], int offset, int len) throws SAXException {
		C_data = C_data + new String(buf, offset, len);
		// System.out.println(C_data);
	}

	public Identification get_identifications() {
		return (identification_);
	}

	public String get_sample() {
		return (sample_.getName());
	}
}
