package fr.inra.pappso.xtandempipeline.sax_parse;

import java.io.InputStream;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.xtandem_param;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationXtandemFile;

public class xtandem_params {

	private static final Logger logger = Logger.getLogger(xtandem_params.class);

	private Vector<xtandem_param> params;

	private Vector<String> groups;

	private InputStream template = this.getClass().getResourceAsStream(
			"/resources/xtandem_model.xml");

	public xtandem_params() throws Exception {
		this.params = new Vector<xtandem_param>();
		this.groups = new Vector<String>();
		this.load_new_param();
	}

	public void load_xml(IdentificationXtandemFile xml) throws Exception {
		logger.info("Load xtandem param from : " + xml.getName());
		new parse_file(new xml_xtandem_param(this), xml);
	}

	private void load_new_param() throws Exception {
		if (template == null)
			throw new MSMSException(
					"Problem accessing resource : xtandem_model.xml");
		new_xtandem_param handler = new new_xtandem_param(this);
		new parse_file(handler, template);
		logger.debug(this.groups.size());
	}

	public void add_params(xtandem_param p) {
		this.params.add(p);
		if (!this.groups.contains(p.getGroup()))
			this.groups.add(p.getGroup());
	}

	public Vector<xtandem_param> get_params_to_group(String g) {
		Vector<xtandem_param> x = new Vector<xtandem_param>();
		for (int i = 0; i < this.params.size(); i++)
			if (this.params.elementAt(i).getGroup().equals(g))
				x.add(this.params.elementAt(i));
		return x;
	}

	public xtandem_param get_param_to(String note_value) {
		for (int i = 0; i < this.params.size(); i++)
			if (this.params.elementAt(i).xmlvalue()
					.equalsIgnoreCase(note_value))
				return (this.params.elementAt(i));
		return (null);
	}

	public Vector<String> get_groups() {
		return (groups);
	}

	public void save_to_xml(IdentificationXtandemFile xml) throws Exception {
		msms_output out = new msms_output();
		out.open(xml.getAbsolutePath());
		out.write("<?xml version=\"1.0\"?>" + "\n");
		out.write("<bioml label=\"example api document\">" + "\n");
		for (int i = 0; i < this.params.size(); i++) {
			out.write(params.elementAt(i).write_to_xml() + "\n");
		}
		out.write("</bioml>" + "\n");
		out.close();
	}

}
