package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.ModifsList;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_peptide_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_protein_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.filter_isotopic_modifs;
import fr.inra.pappso.xtandempipeline.filter_print.print_fasta;
import fr.inra.pappso.xtandempipeline.filter_print.print_pepnovo;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;
import fr.inra.pappso.xtandempipeline.sax_parse.proticdbml;
import fr.inra.pappso.xtandempipeline.sax_parse.MassChroQML;

public class printing_base extends process {

	private File out;

	protected ArrayList<Identification> identifications = XtandemPipelineSession
			.getInstance().getIdentificationList();

	private print_fasta fasta_type;

	private ModifsList isotopic_mods;

	public printing_base(config conf) {
		super(conf);
		isotopic_mods = new ModifsList();
		this.fasta_type = null;
		this.set_max_progress(8);
	}

	protected void processing() throws Exception {
		// Ecrire le process et penser à relever des StopException
		logger.info("Printing to other format");

		msms_output output = new msms_output();

		if (conf.get_export_txt()) {
			this.set_view("Printing text tabulated file results");
			this.specific_printing();
		}

		if (conf.get_export_ods()) {
			this.set_view("Printing Open Document Spreadsheet file results");
			
			OdsDocWriter doc = this.ods_specific_printing(this.getfileout()
					.getAbsolutePath() + ".ods");
			doc.close();
					
		}

		if (conf.get_export_proticdbml()) {
			this.set_view("Printing PROTICdbML file results");
			proticdbml protic = new proticdbml(new File(this.getfileout()
					.getAbsolutePath() + "_protic.xml"));
			for (Identification ident : this.identifications) {
				protic.add_identification(ident);
			}
		}

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_fasta()) {
			this.set_view("Printing fasta file protein result");
			HashSet<Protein> prot_deja_ecris = new HashSet<Protein>();
			output.open(this.getfileout().getAbsolutePath() + ".fasta");
			this.fasta_type = this.fasta_export_choice(output, prot_deja_ecris);
			for (Identification ident : this.identifications) {
				ident.print_result(this.fasta_type);
			}
			output.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_pepnovo()) {
			this.set_view("Printing PepNovo xml file results");
			output.open(this.getfileout().getAbsolutePath() + "_pepnovo.xml");
			output.write("<?xml version=\"1.0\"?>" + "\n"
					+ "<tandem_to_pepnovo>" + "\n");
			for (Identification ident : this.identifications) {
				ident.print_result(new print_pepnovo(output, ident
						.get_samples()));
			}
			output.write("</tandem_to_pepnovo>" + "\n");
			output.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// Print de fdr...
		if ((conf.get_Export_fdr()) & (this.identifications.size() == 1)) {
			this.set_view("Printing FDR");
			output.open(this.getfileout().getAbsolutePath()
					+ "_fdr_peptide.txt");
			calculation_peptide_fdr_distribution calc = new calculation_peptide_fdr_distribution();
			calc.execute(this.identifications.get(0)
					.get_protein_liste());
			calc.print_peptide_fdr_distribution(output);
			output.close();

			output.open(this.getfileout().getAbsolutePath()
					+ "_fdr_protein.txt");
			calculation_protein_fdr_distribution calc2 = new calculation_protein_fdr_distribution();
			calc2.execute(this.identifications.get(0)
					.get_protein_liste());
			calc2.print_protein_fdr_distribution(output);
			output.close();

		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if ((conf.get_export_masschroq())
				& (this.identifications.size() == 1)) {
			this.set_view("Printing MassChroQ xml file results");
			Identification ident = this.identifications.get(0);
			// modification isotopic
			ident.filtered_match(new filter_isotopic_modifs(this.isotopic_mods));
			MassChroQML xml = new MassChroQML(new File(this.getfileout()
					.getAbsolutePath() + ".masschroqML"));
			// transfert des isotopes
			xml.set_isotopes_mods(this.isotopic_mods);
			xml.set_identification(ident);
		}

		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	protected OdsDocWriter ods_specific_printing(String filename) throws Exception {
		return null;
	}

	protected void specific_printing() throws Exception {

	}

	private print_fasta fasta_export_choice(msms_output out,
			HashSet<Protein> prot_deja_ecris) {
		if (this.fasta_type == null) {
			this.fasta_type = new print_fasta();
			System.out.println("fasta export reinitialise");
		}
		this.fasta_type.setProt_deja_ecris(prot_deja_ecris);
		this.fasta_type.setOut(out);
		return (this.fasta_type);
	}

	public void setfileout(File out) {
		this.out = out;
	}

	public File getfileout() {
		return this.out;
	}

	public void setIsotopic_mods(ModifsList isotopic_mods) {
		this.isotopic_mods = isotopic_mods;
	}

	public ModifsList getIsotopic_mods() {
		return isotopic_mods;
	}

	public void setFasta_type(print_fasta fasta_type) {
		this.fasta_type = fasta_type;
		System.out.println(this.fasta_type.toString());
	}

	public print_fasta getFasta_type() {
		return fasta_type;
	}

}
