package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationXtandemFile;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem_params;

public class xtandem_analysis extends process {

	private File[] xtandem;

	private File[] databases;

	private File save_dir;

	private IdentificationXtandemFile model;

	public xtandem_analysis(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Xtandem analysis start");
		// creation du fichier database
		File databasexml = new File(this.save_dir.getAbsolutePath()
				+ System.getProperty("file.separator") + "database.xml");
		msms_output out = new msms_output();
		out.open(databasexml.getAbsolutePath());
		out.write("<?xml version=\"1.0\"?>" + "\n");
		out.write("<bioml label=\"x! taxon-to-file matching list\">" + "\n");
		out.write("<taxon label=\"usedefined\">" + "\n");
		for (int j = 0; j < databases.length; j++) {
			out.write("<file format=\"peptide\" URL=\""
					+ databases[j].getAbsolutePath() + "\" />" + "\n");
		}
		out.write("</taxon>" + "\n");
		out.write("</bioml>" + "\n");
		out.close();
		File infile = new File(this.save_dir.getAbsolutePath()
				+ System.getProperty("file.separator") + "infile.xml");

		// Correction du fichier de parametres
		logger.info("Corrected xtandem preset error");
		xtandem_params params = new xtandem_params();
		params.load_xml(model);
		params.save_to_xml(model);

		for (int i = 0; i < xtandem.length; i++) {
			logger.info("Xtandem analysis on : " + xtandem[i].getName());
			File result_out = new File(this.get_out(xtandem[i]));
			// parsing
			// creation du fichier infile
			out.open(infile.getAbsolutePath());
			out.write("<?xml version=\"1.0\"?>" + "\n");
			out.write("<bioml label=\"" + xtandem[i].getName() + "\">" + "\n");
			out.write("<note type=\"heading\">Paths</note>" + "\n");
			out.write("<note type=\"input\" label=\"list path, default parameters\">"
					+ model.getAbsolutePath() + "</note>" + "\n");
			out.write("<note type=\"input\" label=\"list path, taxonomy information\">"
					+ databasexml.getAbsolutePath() + "</note>" + "\n");
			out.write("<note type=\"input\" label=\"spectrum, path\">"
					+ xtandem[i].getAbsolutePath() + "</note>" + "\n");
			out.write("<note type=\"heading\">Protein general</note>" + "\n");
			out.write("<note type=\"input\" label=\"protein, taxon\">usedefined</note>"
					+ "\n");
			out.write("<note type=\"heading\">Output</note>" + "\n");
			out.write("<note type=\"input\" label=\"output, path\">"
					+ result_out.getAbsolutePath() + "</note>" + "\n");
			out.write("</bioml>" + "\n");
			out.close();

			// creation des fichiers
			base_shell pipe = new base_shell();
			pipe.set_commande(new String[] {
					conf.getXtandem_link().getAbsolutePath(),
					infile.getAbsolutePath() });
			pipe.start_process();
			while (pipe.status()) {
				Thread.currentThread();
				Thread.sleep(100);
				if (this.isStop()) {
					pipe.stop_process();
					throw new StopException("");
				}
				this.set_view("Analyse file : " + (i + 1) + "/"
						+ xtandem.length + "\n" + xtandem[i].getName() + "\n"
						+ pipe.get_Stdout());
				// System.out.println(pipe.getStatus());

			}
			if (pipe.get_error() != null)
				throw pipe.get_error();
			if (!result_out.exists())
				throw new MSMSException("Analysis of '" + xtandem[i].getName()
						+ " give no result'\n" + pipe.get_Stdout());

			logger.info("finish");
			this.add_current_progress();
		}
		databasexml.delete();
		infile.delete();

	}

	public void setXtandem(File[] xtandem) {
		this.xtandem = xtandem;
		this.set_max_progress(xtandem.length);
	}

	public File[] getXtandem() {
		return xtandem;
	}

	public void setDatabases(File[] files) {
		this.databases = files;
	}

	public File[] getDatabases() {
		return databases;
	}

	public void setSave_dir(File save_dir) {
		this.save_dir = save_dir;
	}

	public File getSave_dir() {
		return save_dir;
	}

	public void setModel(IdentificationXtandemFile model) {
		this.model = model;
	}

	public IdentificationXtandemFile getModel() {
		return model;
	}

	private String get_out(File out) {
		String basename = out.getName();
		basename = basename.replaceFirst(".mzXML$", "");
		basename = basename.replaceFirst(".mgf$", "");
		basename = basename.replaceFirst(".mzData$", "");
		basename = basename.replaceFirst(".mzML$", "");
		basename = basename.replaceFirst(".pkl$", "");
		basename = basename.replaceFirst(".xml$", "");

		return (this.getSave_dir().getAbsolutePath()
				+ System.getProperty("file.separator") + basename + "." + "xml");
	}

}
