package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;

public class Peptide implements Comparable<Peptide> {
	private static final Logger logger = Logger.getLogger(Peptide.class);

	final static float phospho_mass = (float) 79.96633;

	private IdentificationDataSource sample_file;

	private int scan_in_xtandem;

	private HashSampleScan hashSampleScan;

	private float RT;

	private float mhplus_obser;

	private float mhplus_theo;

	private float deltamass;

	private String sequence;

	private String pre;

	private String post;

	private int start;

	private int stop;

	private int charge;

	private float evalue;

	private float hypercorr;

	private ArrayList<Modifs> modifList;

	private boolean validate;

	private ArrayList<Double> mz_values;

	private ArrayList<Double> intensity_values;

	public int compareTo(Peptide p) {
		int i = this.hashSampleScan.compareTo(p.getSampleScan());
		if (i != 0)
			return (i);
		float delta = this.evalue - p.get_evalue();
		if (delta > 0) {
			return (1);
		} else if (delta < 0) {
			return (-1);
		} else {
			return (0);
		}
	}

	// constructeur
	public Peptide() {
		this.modifList = new ArrayList<Modifs>();
		this.RT = 0;
		this.validate = true;
		this.scan_in_xtandem = 0;
		this.hashSampleScan = null;
	}

	public boolean isValidate() {
		return validate;
	}

	public void setValidate(boolean validate) {
		this.validate = validate;
	}

	public int get_sample_scan_cle() {
		return (this.hashSampleScan.hashCode());
	}

	public String get_sample_scan_sequencepeptide_cle() {
		String cle = this.get_sample() + "_" + this.get_scan() + "_"
				+ this.get_sequence() + "_" + this.get_Modifs_formater();
		return (cle);
	}

	public String get_unique_cle() {
		String cle = this.get_sample() + "_" + this.get_scan() + "_"
				+ this.start + "_" + this.get_sequence_formater() + "_"
				+ this.stop + "_" + this.get_Modifs_formater();
		return (cle);
	}

	public void set_sample_id(MsRun msRun, IdentificationDataSource origin) {
		this.sample_file = msRun.addFile(origin);
		if (this.hashSampleScan == null) {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(-1, msRun);
		} else {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(hashSampleScan.getScan(), msRun);
		}
	}

	public void set_scan(int i) {
		if (this.hashSampleScan == null) {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(i, null);
		} else {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(i, hashSampleScan.getMsRun());
		}
		// on conserve un lien vers le numéro de scan xtandem
		if (this.scan_in_xtandem == 0)
			this.scan_in_xtandem = i;
	}

	public void set_charge(int i) {
		this.charge = i;
	}

	public void set_RT(String i) {
		i = i.replaceAll("PT", "");
		i = i.replaceAll("S", "");
		try {
			this.RT = Float.valueOf(i).floatValue();
		} catch (Exception e) {
			this.RT = 0;
		}
	}

	public void set_mhplus_obser(float i) {
		this.mhplus_obser = i;
	}

	public void set_mhplus_theo(float i) {
		this.mhplus_theo = i;
	}

	public void set_pre(String i) {
		this.pre = i;
	}

	public void set_post(String i) {
		this.post = i;
	}

	public void set_start(int i) {
		this.start = i;
	}

	public void set_stop(int i) {
		this.stop = i;
	}

	public void set_deltamass(float f) {
		this.deltamass = f;
	}

	public void set_evalue(float f) {
		this.evalue = f;
	}

	public void set_hypercorr(float f) {
		this.hypercorr = f;
	}

	public void set_Modifs(String AA, int posi_in_prot, float modvalue) {
		int posi_corrected = posi_in_prot - this.start + 1;
		Modifs mod = new Modifs(AA, posi_corrected, modvalue);
		mod.setPosi_in_prot(posi_in_prot);
		this.modifList.add(mod);
	}

	public void set_sequence(String S) {
		this.sequence = S;
	}

	public String get_sample() {
		return (hashSampleScan.getMsRun().getSampleName());
	}

	public int get_scan() {
		return (hashSampleScan.getScan());
	}

	public int get_charge() {
		return (charge);
	}

	public float get_RT() {
		int temp = (int) (this.RT / 0.6);
		Float t = (float) temp / 100;
		return (t);
	}

	public int get_RT_in_sec() {
		return ((int) this.RT);
	}

	public float get_mhplus_obser() {
		return (this.mhplus_obser);
	}

	public float get_mhplus_theo() {
		return (this.mhplus_theo);
	}

	public String get_pre() {
		return (this.pre);
	}

	public String get_post() {
		return (this.post);
	}

	public int get_start() {
		return (this.start);
	}

	public int get_stop() {
		return (this.stop);
	}

	public float get_deltamass() {
		return (this.deltamass);
	}

	public float get_evalue() {
		return (this.evalue);
	}

	public float get_hypercorr() {
		return (this.hypercorr);
	}

	public ArrayList<Modifs> get_Modifs() {
		return (this.modifList);
	}

	public String get_Modifs_formater() {
		StringBuilder format = new StringBuilder();
		if (modifList.size() >= 1) {
			Modifs mod = modifList.get(0);
			format.append(mod.get_formater_value());
		}
		if (modifList.size() >= 2) {
			for (int i = 1; i < modifList.size(); i++) {
				Modifs mod = modifList.get(i);
				format.append(" - " + mod.get_formater_value());
			}
		}
		format.append(" ");
		return (format.toString());
	}

	public Float get_mass_Modifs_without_isotope(boolean removeMS3) {
		Float f = (float) 0;
		for (Modifs mod : modifList) {
			if (!mod.isIsotopic()) {
				if (mod.is_phopho_modifs() && removeMS3)
					f += Peptide.phospho_mass;
				else
					f += mod.get_modvalue();
			}
		}
		return (f);
	}

	public Float get_mh_theo_without_isotope(boolean correctedMS3) {
		Float f = this.mhplus_theo;
		for (Modifs mod : modifList) {
			if (mod.isIsotopic()) {
				f -= mod.get_modvalue();
			} else if (mod.is_phopho_modifs() && correctedMS3) {
				f -= mod.get_modvalue();
				f += Peptide.phospho_mass;
			}
		}
		return (f);
	}

	public boolean is_phopho_peptide() {
		boolean phospho = false;
		for (Modifs mod : modifList) {
			if (mod.is_phopho_modifs())
				phospho = true;
		}
		return (phospho);
	}

	public ArrayList<Modifs> get_phospho_Modifs() {
		ArrayList<Modifs> phospho = new ArrayList<Modifs>();
		for (Modifs mod : modifList) {
			if (mod.is_phopho_modifs())
				phospho.add(mod);
		}
		return (phospho);
	}

	public String get_sequence() {
		return (sequence);
	}

	public String get_LI_sequence() {
		String seq = this.sequence.replaceAll("L", "I");
		return (seq);
	}

	public String get_LI_sequence_modifs() {
		String seq = this.get_LI_sequence();
		seq = seq + "-"
				+ this.get_mass_Modifs_without_isotope(false).intValue();
		return (seq);
	}

	public String get_LI_sequenceModifsWithoutMs3() {
		String seq = this.get_LI_sequence();
		seq = seq + "-" + this.get_mass_Modifs_without_isotope(true).intValue();
		return (seq);
	}

	public String get_sequence_formater() {
		String format = "" + this.pre + "." + this.sequence + "." + this.post;
		return (format);
	}

	public float get_mz_theo() {
		float mz = (float) ((this.get_mhplus_theo() + ((this.get_charge() - 1) * 1.007825)) / this
				.get_charge());
		return mz;
	}

	public String get_phospho_Modifs_position() {
		ArrayList<Modifs> pho = this.get_phospho_Modifs();
		String S = "";
		for (Modifs posi : pho) {
			S += " " + posi.getPosi_in_prot();
		}
		S.replaceFirst(" ", "");
		return S;
	}

	public IdentificationDataSource getSample_file() {
		return sample_file;
	}

	public int getScan_in_xtandem() {
		return scan_in_xtandem;
	}

	public void setIntensity_values(String C_data, float factor) {
		C_data = C_data.replaceAll("\n", " ");
		C_data = C_data.replaceAll("^\\s+", "");
		C_data = C_data.replaceAll("\\s+$", "");
		String[] values = C_data.split(" ");
		this.intensity_values = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++)
			intensity_values.add(Double.valueOf(values[i]).doubleValue()
					* factor);
	}

	public void setIntensity_values(double[] intenArr) {
		this.intensity_values = new ArrayList<Double>();
		for (int i = 0; i < intenArr.length; i++)
			intensity_values.add(Double.valueOf(intenArr[i]).doubleValue());
	}

	public ArrayList<Double> getIntensity_values() {
		return intensity_values;
	}

	public void setMz_values(String C_data) {
		C_data = C_data.replaceAll("\n", " ");
		C_data = C_data.replaceAll("^\\s+", "");
		C_data = C_data.replaceAll("\\s+$", "");
		String[] values = C_data.split(" ");
		this.mz_values = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++)
			mz_values.add(Double.valueOf(values[i]).doubleValue());
	}

	public void setMz_values(double[] mzArr) {
		this.mz_values = new ArrayList<Double>();
		for (int i = 0; i < mzArr.length; i++)
			mz_values.add(Double.valueOf(mzArr[i]).doubleValue());
	}

	public ArrayList<Double> getMz_values() {
		return mz_values;
	}

	public MsRun getMsRun() {
		return hashSampleScan.getMsRun();
	}

	public HashSampleScan getSampleScan() {
		if (hashSampleScan.getMsRun() == null) {
			return null;
		}
		if (hashSampleScan.getScan() == -1) {
			return null;
		}
		return hashSampleScan;
	}

	public HashPeptide getHashPeptide() throws Exception {
		HashSampleScan hashScan = getSampleScan();
		if (hashScan == null) {
			throw new Exception("hashSampleScan is null for this peptide");
		}
		return XtandemPipelineSession.getInstance().getHashPeptideInstance(
				hashScan, this.get_LI_sequence());
	}

	public boolean containsPostion(Integer position) throws Exception {
		if (position == null) {
			String error = "position is null";
			logger.debug(error);
			throw new Exception(error);
		}
		if (this.start > position) {
			return false;
		}
		if (position > this.stop) {
			return false;
		}
		return true;
	}

	public HashSampleScan getHashSampleScan() {
		return this.hashSampleScan;
	}

	public String get_Modifs_formater_without_isotope() {
		StringBuilder format = new StringBuilder();
		for (Modifs mod : modifList) {
			if (!mod.isIsotopic()) {
				format.append(" " + mod.get_formater_value());
			}
		}
		format.append(" ");
		return (format.toString());
	}

}
