package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;
import java.util.TreeSet;

import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class print_peptide_list_to_phosphopeptide extends print_base {

	public print_peptide_list_to_phosphopeptide(msms_output out) {
		super(out);
		logger.info("Print Peptide liste file in phospho mode");
	}

	public void print_header() throws Exception {
		String S = "Groupe" + "\t" + "Description" + "\t" + "Sample" + "\t"
				+ "Scan" + "\t" + "Rt" + "\t" + "Sequence" + "\t" + "Best Position"
				+ "\t" + "Phosphopeptide" + "\t" + "All Position" + "\t"+ "Modifs" + "\t"+ "Used" + "\t" + "on a total of"
				+ "\t" + "Sub-groups" + "\t" + "E-value" + "\t" + "Charge"
				+ "\t" + "MH+ Obs" + "\t" + "MH+ theo" + "\t" + "DeltaMH+"
				+ "\t" + "Delta-ppm" + "\n";
		out.write(S);
	}

	public void print_group_information(Group group) throws Exception {
		for (HashSampleScan hash : group.getHashSampleScanSet()
				.getHashSampleScanList()) {
			SubGroupSet set = group.getSubGroupContaining(hash);
			ArrayList<SubGroup> sgList = set.getSubGroupList();
			HashSampleScanSetPhospho phospho =  (HashSampleScanSetPhospho) sgList.get(0).getHashSampleScanSetProtList().get(0);
			Match top = phospho.getMatch();
			Peptide pep = top.getBestPeptideToHashSampleScan(hash);
			StringBuilder S = new StringBuilder();

			// on rempli
			S.append(group.getNumber() + "\t");
			if (set.size() == 1)
				S.append(top.get_protein_match().get_description());
			S.append("\t" + hash.getMsRun().getSampleName());
			S.append("\t" + hash.getScan());
			S.append("\t" + pep.get_RT());
			S.append("\t" + pep.get_sequence());
			// position
			S.append("\t" + pep.get_phospho_Modifs_position());
			
			// Comparaison des positions lié à l'incertitude
			TreeSet<Integer> posi = phospho.getAllPositionIncluderedon(hash);
			ArrayList<Modifs> phosphoModifs = pep.get_phospho_Modifs();

			// Multi
			if (phosphoModifs.size() >= 1) {
				if (phosphoModifs.size() > 1) {
					S.append("\t" + "Multi" + "\t");
				} else {
					S.append("\t" + "Mono" + "\t");
				}
				S.append("\"");
				for (Integer pos : posi)
					S.append(pos + " ");
				S.append("\"");
			} else {
				S.append("\t" + "No" + "\t" + "\"-\"");
			}

			S.append("\t" + pep.get_Modifs_formater());
			
			if (set.size() > 1)
				S.append("\t" + set.size() + "\t"
						+ group.getSubGroupSet().size());
			else
				S.append("\t" + "\"-\"" + "\t" + "\"-\"");
			S.append("\t");

			
			boolean second = false;
			for (SubGroup sub : sgList) {
				if (second) {
					S.append(" ");
				}
				S.append(this.get_subgroup_number(group, sub));
				second = true;
			}

			S.append("\t" + pep.get_evalue());
			S.append("\t" + pep.get_charge());
			S.append("\t" + pep.get_mhplus_obser());
			S.append("\t" + pep.get_mhplus_theo());
			S.append("\t" + pep.get_deltamass());
			S.append("\t"
					+ ((pep.get_deltamass() / pep.get_mhplus_theo()) * 1000000));
			S.append("\n");
			out.write(S.toString());
		}
		out.write("\n");
	}

}
